/**
 * @file package.c Package information
 * 
 * $Id: package.c,v 1.11 2002/10/21 19:15:40 mhearn Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public List
 * as published by the Free Software Foundation; either version 2
 * of the List, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public List for more details.
 *
 * You should have received a copy of the GNU General Public List
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

#include <libxml/parser.h>

static void
packageStartElement(void *userData, const char *name, const char **attrs)
{
	PackageParserState *state = (PackageParserState *)userData;

	if (state->tag != PACKAGE_NONE)
	{
		fprintf(stderr, "Parse error in XML!\n");
		exit(1);
	}
	
	if (!strcmp(name, "info"))
	{
		state->tag = PACKAGE_NONE;
		state->inInfo = 1;
	}
	else if (!strcmp(name, "checks"))
	{
		state->tag = PACKAGE_NONE;
		state->inChecks = 1;
	}
	else if (!strcmp(name, "check"))
	{
		Check *check = newCheck();

		addCheck(state->package, check);

		state->check = check;
	}
	else if (!strcmp(name, "name"))        state->tag = PACKAGE_NAME;
	else if (!strcmp(name, "description")) state->tag = PACKAGE_DESCRIPTION;
	else if (!strcmp(name, "file"))        state->tag = PACKAGE_FILE;
	else if (!strcmp(name, "macro"))       state->tag = PACKAGE_MACRO;
	else if (!strcmp(name, "define"))      state->tag = PACKAGE_DEFINE;
	else if (!strcmp(name, "cflags"))      state->tag = PACKAGE_CFLAGS;
	else if (!strcmp(name, "libs"))        state->tag = PACKAGE_LIBS;
}

static void
packageEndElement(void *userData, const char *name)
{
	PackageParserState *state = (PackageParserState *)userData;

	if (!strcmp(name, "check"))
	{
		if (state->list != NULL)
		{
			char isDefault;
			ListItem *item;

			isDefault = inArray((const char **)state->defaults,
								state->check->name, state->defaultCount);
			
			item = newDialogListItem(state->check->name,
									 state->check->description, isDefault);
			dialogListAddItem(state->list, item);
		}

		state->check = NULL;
	}
	else if (!strcmp(name, "info"))   state->inInfo = 0;
	else if (!strcmp(name, "checks")) state->inChecks = 0;
	
	state->tag = PACKAGE_NONE;
}

static void
packageCharacters(void *userData, const char *ch, int len)
{
	PackageParserState *state = (PackageParserState *)userData;
	
	char *data = (char *)malloc((len + 1) * sizeof(char));

	strncpy(data, ch, len);
	data[len] = '\0';

	if (isWhitespace(data))
	{
		free(data);
		
		return;
	}

	if (state->inInfo)
	{
		Package *package = state->package;
		
		switch (state->tag)
		{
			case PACKAGE_NAME:        package->name        = data; break;
			case PACKAGE_DESCRIPTION: package->description = data; break;

			default:
				free(data);
				break;
		}
	}
	else if ((state->inChecks) && (state->check != NULL))
	{
		Check *check = state->check;

		switch (state->tag)
		{
			case PACKAGE_NAME:        check->name        = data; break;
			case PACKAGE_DESCRIPTION: check->description = data; break;
			case PACKAGE_FILE:        check->file        = data; break;
			case PACKAGE_MACRO:       check->macro       = data; break;
			case PACKAGE_DEFINE:      check->define      = data; break;
			case PACKAGE_CFLAGS:      check->cflags      = data; break;
			case PACKAGE_LIBS:        check->libs        = data; break;

			default:
				free(data);
				break;
		}
	}
	else
		free(data);
}

static xmlSAXHandler packageHandler = {
	NULL,                                     /* internalSubset        */
	NULL,                                     /* isStandalone          */
	NULL,                                     /* hasInternalSubset     */
	NULL,                                     /* hasExternalSubset     */
	NULL,                                     /* resolveEntity         */
	NULL,                                     /* getEntity             */
	NULL,                                     /* entityDecl            */
	NULL,                                     /* notationDecl          */
	NULL,                                     /* attributeDecl         */
	NULL,                                     /* elementDecl           */
	NULL,                                     /* unparsedEntityDecl    */
	NULL,                                     /* setDocumentLocator    */
	NULL,                                     /* startDocument         */
	NULL,                                     /* endDocument           */
	(startElementSAXFunc)packageStartElement, /* startElement          */
	(endElementSAXFunc)packageEndElement,     /* endElement            */
	NULL,                                     /* reference             */
	(charactersSAXFunc)packageCharacters,     /* characters            */
	NULL,                                     /* ignorableWhitespace   */
	NULL,                                     /* processingInstruction */
	NULL,                                     /* comment               */
	(warningSAXFunc)xmlWarning,               /* warning               */
	(errorSAXFunc)xmlError,                   /* error                 */
	(fatalErrorSAXFunc)xmlFatalError          /* fatalError            */
};

Package *
newPackage()
{
	Package *pkg;

	pkg = (Package *)malloc(sizeof(Package));

	pkg->name        = NULL;
	pkg->description = NULL;
	pkg->checks      = NULL;
	pkg->lastCheck   = NULL;
	pkg->next        = NULL;
	pkg->prev        = NULL;

	return pkg;
}

Package *
loadPackage(const char *packageName, DialogList *list,
			char **defaults, int defaultCount)
{
	PackageParserState state;
	Package *pkg;
	char *file;

	file = (char *)malloc((strlen(data_dir) + strlen(packageName) + 15) *
						  sizeof(char));
	
	strcpy(file, data_dir);
	strcat(file, "/packages/");
	strcat(file, packageName);
	strcat(file, ".xml");

	pkg = newPackage();

	state.tag          = PACKAGE_NONE;
	state.inInfo       = 0;
	state.inChecks     = 0;
	state.list         = list;
	state.package      = pkg;
	state.check        = NULL;
	state.defaults     = defaults;
	state.defaultCount = defaultCount;

	parseXml(&packageHandler, &state, file);

	free(file);
	
	return pkg;
}

void
freePackage(Package *pkg)
{
	Check *curCheck, *nextCheck;
	
	if (pkg == NULL)
		return;

	if (pkg->name        != NULL) free(pkg->name);
	if (pkg->description != NULL) free(pkg->description);

	for (curCheck = pkg->checks; curCheck != NULL; curCheck = nextCheck)
	{
		nextCheck = curCheck->next;
		freeCheck(curCheck);
	}

	free(pkg);
}

Package *
copyPackage(Package *src)
{
	Package *dest;

	if (src == NULL)
		return NULL;

	dest = newPackage();
	
	dest->name        = (src->name        ? strdup(src->name)        : NULL);
	dest->description = (src->description ? strdup(src->description) : NULL);

	dest->checks    = NULL;
	dest->lastCheck = NULL;

	return dest;
}

int
getCheckCount(Package *pkg)
{
	Check *check;
	int count = 0;
	
	if (pkg == NULL)
		return 0;

	for (check = pkg->checks; check != NULL; check = check->next)
	{
		count++;
	}
	
	return count;
}

int
packageToArray(Package *pkg, char ***array)
{
	Check *check;
	int count, i = 0;
	
	if (pkg == NULL)
		return 0;

	count = getCheckCount(pkg);

	if (count == 0)
	{
		*array = NULL;
		return 0;
	}
	
	*array = (char **)malloc(count * sizeof(char *));

	for (check = pkg->checks; check != NULL; check = check->next)
	{
		if (check->name == NULL)
			(*array)[i++] = NULL;
		else
			(*array)[i++] = strdup(check->name);
	}

	return count;
}

Check *
newCheck()
{
	Check *check;

	check = (Check *)malloc(sizeof(Check));

	check->name        = NULL;
	check->description = NULL;
	check->file        = NULL;
	check->macro       = NULL;
	check->define      = NULL;
	check->cflags      = NULL;
	check->libs        = NULL;
	check->next        = NULL;

	return check;
}

void
freeCheck(Check *check)
{
	if (check == NULL)
		return;

	if (check->name        != NULL) free(check->name);
	if (check->description != NULL) free(check->description);
	if (check->file        != NULL) free(check->file);
	if (check->macro       != NULL) free(check->macro);
	if (check->define      != NULL) free(check->define);
	if (check->cflags      != NULL) free(check->cflags);
	if (check->libs        != NULL) free(check->libs);

	free(check);
}

Check *
getCheckWithName(Package *pkg, const char *name)
{
	Check *check;

	if (pkg == NULL || name == NULL)
		return NULL;

	for (check = pkg->checks; check != NULL; check = check->next)
	{
		if (!strcmp(name, check->name))
			return check;
	}

	return NULL;
}

Check *
getFirstCheck(Package *pkg)
{
	if (pkg == NULL)
		return NULL;

	return pkg->checks;
}

Check *
getNextCheck(Check *check)
{
	if (check == NULL)
		return NULL;

	return check->next;
}

Check *
copyCheck(Check *src)
{
	Check *dest;

	if (src == NULL)
		return NULL;
	
	dest = newCheck();

	dest->name        = (src->name        ? strdup(src->name)        : NULL);
	dest->description = (src->description ? strdup(src->description) : NULL);
	dest->file        = (src->file        ? strdup(src->file)        : NULL);
	dest->macro       = (src->macro       ? strdup(src->macro)       : NULL);
	dest->define      = (src->define      ? strdup(src->define)      : NULL);
	dest->cflags      = (src->cflags      ? strdup(src->cflags)      : NULL);
	dest->libs        = (src->libs        ? strdup(src->libs)        : NULL);

	dest->next = NULL;
	dest->prev = NULL;
	
	return dest;
}

void
addCheck(Package *pkg, Check *check)
{
	if (pkg == NULL || check == NULL)
		return;

	if (pkg->checks == NULL)
		pkg->checks = check;
	
	check->prev = pkg->lastCheck;
	
	if (pkg->lastCheck != NULL)
		pkg->lastCheck->next = check;

	pkg->lastCheck = check;
}

