/**
 * @file projectinfo.c Project information
 * 
 * $Id: projectinfo.c,v 1.9 2001/12/31 09:26:55 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

ProjectInfo *
newProjectInfo()
{
	ProjectInfo *project;

	project = (ProjectInfo *)malloc(sizeof(ProjectInfo));

	project->name    = NULL;
	project->version = NULL;
	project->license = NULL;
	project->summary = NULL;
	project->group   = NULL;
	project->dir     = NULL;

	project->languages = (Languages *)malloc(sizeof(Languages));
	
	project->packages    = NULL;
	project->lastPackage = NULL;

	return project;
}

void
freeProjectInfo(ProjectInfo *project)
{
	Package *curPackage, *nextPackage;
	
	if (project == NULL)
		return;

	if (project->name    != NULL) free(project->name);
	if (project->version != NULL) free(project->version);
	if (project->license != NULL) free(project->license);
	if (project->summary != NULL) free(project->summary);
	if (project->group   != NULL) free(project->group);
	if (project->dir     != NULL) free(project->dir);

	for (curPackage = project->packages;
		 curPackage != NULL;
		 curPackage = nextPackage)
	{
		nextPackage = curPackage->next;
		freePackage(curPackage);
	}

	free(project->languages);

	free(project);
}

Package *
getPackageWithName(ProjectInfo *project, const char *name)
{
	Package *curPackage;

	if (project == NULL || name == NULL)
		return NULL;

	for (curPackage = project->packages;
		 curPackage != NULL;
		 curPackage = curPackage->next)
	{
		if (curPackage->name != NULL && !strcmp(curPackage->name, name))
			return curPackage;
	}

	return NULL;
}

void
requestProjectName(ProjectInfo *project)
{
	project->name =
		dialogInputBox("Project Name",
					   "What is the name of your project?",
					   project->name);
}

void
requestProjectVersion(ProjectInfo *project)
{
	project->version =
		dialogInputBox("Project Version",
					   "What version?",
					   (project->version ? project->version : "0.1.0"));
}

void
requestProjectLicense(ProjectInfo *project)
{
	DialogList *list;

	list = newDialogList("License",
						 "What license are you planning on using?");

	xmlAddListItems(list, "licenses.xml");

	project->license = dialogRenderRadioList(list);

	freeDialogList(list);
}

void
requestProjectSummary(ProjectInfo *project)
{
	project->summary =
		dialogInputBox("Project Summary",
					   "Please give a short description of your project.",
					   project->summary);
}

void
requestProjectGroup(ProjectInfo *project)
{
	DialogList *list;

	list = newDialogList("Project Group",
						 "Pick the group that your project belongs to.");

	xmlAddListItems(list, "groups.xml");
	
	for (;;)
	{
		project->group = dialogRenderRadioList(list);

		if (strlen(project->group) > 0)
			break;
	}

	freeDialogList(list);
}

void
requestProjectLanguage(ProjectInfo *project)
{
	DialogList *list;
	char **langs;
	int count, i;

	list = newDialogList("Language",
						 "What language will this project primarily be "
						 "written in?");
	
	dialogListAddItem(list, newDialogListItem("C",       NULL, 1));
	dialogListAddItem(list, newDialogListItem("C++" ,    NULL, 0));
	dialogListAddItem(list, newDialogListItem("Fortran", NULL, 0));
	dialogListAddItem(list, newDialogListItem("YACC",    NULL, 0));
	dialogListAddItem(list, newDialogListItem("Lex",     NULL, 0));
	dialogListAddItem(list, newDialogListItem("AWK",     NULL, 0));

	count = dialogRenderCheckList(list, &langs);

	if (count == 0)
	{
		freeDialogList(list);
		return;
	}
	
	for (i = 0; i < count; i++)
	{
		char *lang = langs[i];

		if ((!project->languages->c) && (!strcmp(lang, "C")))
		{
			project->languages->c = 1;
		}
		else if ((!project->languages->cpp) && (!strcmp(lang, "C++")))
		{
			project->languages->cpp = 1;
		}
		else if ((!project->languages->fortran) && (!strcmp(lang, "Fortran")))
		{
			project->languages->fortran = 1;
		}
		else if ((!project->languages->yacc) && (!strcmp(lang, "YACC")))
		{
			project->languages->yacc = 1;
		}
		else if ((!project->languages->lex) && (!strcmp(lang, "Lex")))
		{
			project->languages->lex = 1;
		}
		else if ((!project->languages->awk) && (!strcmp(lang, "AWK")))
		{
			project->languages->awk = 1;
		}

		free(lang);
	}
	
	free(langs);

	freeDialogList(list);
}

void
requestProjectDirectory(ProjectInfo *project)
{
	char currentDir[512];
	char *defaultDir = NULL;

	strcpy(currentDir, "\0");
	
	if (project->dir == NULL)
	{
		getcwd(currentDir, 512);
		
		defaultDir = (char *)malloc((strlen(currentDir) +
									 strlen(project->name) + 2) *
									sizeof(char));
	
		strcpy(defaultDir, currentDir);
		strcat(defaultDir, "/");
		strcat(defaultDir, project->name);

		project->dir = defaultDir;
	}

	for (;;)
	{
		struct stat statinfo;
		
		project->dir =
			dialogInputBox("Path",
						   "Where do you want the source tree to reside?",
						   project->dir);

		if (stat(project->dir, &statinfo) == 0)
		{
			if (!S_ISDIR(statinfo.st_mode))
			{
				dialogMsgBox("That is not a directory!");
			}
			else
			{
				char *confFile;
				
				confFile = (char *)malloc((strlen(project->dir) + 14) *
										   sizeof(char));
				
				strcpy(confFile, project->dir);
				strcat(confFile, "/config.genst");
				
				if (fileExists(confFile))
				{
					/* TODO: Allow loading from the file. */

					if (dialogYesNo(NULL,
					 "genst has already generated a tree in this directory.\n"
					 "Generating here will overwrite the configure.in,\n"
					 "Makefile.am, etc.\n\n"
					 "Do you really want to continue?"))
					{
						break;
					}
				}

				free(confFile);

				if (dialogYesNo(NULL,
					"There is already a directory with this name, but it\n"
					"does not appear to be generated by genst. Do you still\n"
					"want to use this directory? (Not recommended)\n"))
				{
					break;
				}
			}
			
		}
		else
		{
			if (errno == ENOENT)
				break;
			
			switch (errno)
			{
				case ENOTDIR:
				{
					dialogMsgBox("One of the directories in the path "
								 "is not a directory.");
					break;
				}

				case ELOOP:
				{
					dialogMsgBox("Too many symbolic links were encountered "
								 "while traversing the path.");
					break;
				}

				case EACCES:
				{
					dialogMsgBox("Permission denied! :(");
					break;
				}

				case ENOMEM:
				{
					dialogMsgBox("Out of memory! Sorry...");
					exit(1);
				}

				case ENAMETOOLONG:
				{
					dialogMsgBox("The filename is too long.");
					break;
				}

				default:
				{
					dialogMsgBox("Unknown error!");
					break;
				}
			}
		}
	}

	if (defaultDir != NULL)
		free(defaultDir);
}

void
requestProjectTarget(ProjectInfo *project)
{
	DialogList *list;
	char *target;
	
	list = newDialogList("Targets", "What will you be generating?");
	
	dialogListAddItem(list, newDialogListItem("Executable", "Executable", 1));
	dialogListAddItem(list, newDialogListItem("Library",    "Library",    0));
	dialogListAddItem(list, newDialogListItem("Both",
											  "Executable and Library",   0));

	target = dialogRenderRadioList(list);

	freeDialogList(list);

	if (target == NULL)
		return;

	if (!strcmp(target, "Executable"))   project->target = TARGET_EXECUTABLE;
	else if (!strcmp(target, "Library")) project->target = TARGET_LIBRARY;
	else if (!strcmp(target, "Both"))    project->target = TARGET_BOTH;
	else
	{
		/* Should never happen... */
		free(target);

		fprintf(stderr, "Invalid target!\n");
		exit(1);
	}

	free(target);
}

void
requestPackages(ProjectInfo *project)
{
	DialogMenu *menu;
	char *menuSel = NULL;
	
	menu = newDialogMenu("Packages",
						 "Select every package that configure.in should\n"
						 "check for.");

	addPackages(menu);
	
	dialogAddMenuSep(menu);
	dialogAddMenuItem(menu, newDialogMenuItem("Generate",
											  "Generate the source tree"));
	dialogAddMenuItem(menu, newDialogMenuItem("Quit", "Quit genst"));

	for (;;)
	{
		dialogSetMenuDefault(menu, menuSel);
		
		if (menuSel != NULL)
			free(menuSel);

		menuSel = dialogRenderMenu(menu);
		
		if (!strcmp(menuSel, "Quit"))
		{
			if (confirmQuit())
			{
				freeDialogMenu(menu);
				free(menuSel);

				quit();
			}
		}
		else if (!strcmp(menuSel, "Generate"))
		{
			generate(project);
			break;
		}
		else
		{
			requestChecks(project, menuSel);
		}
	}

	free(menuSel);
	freeDialogMenu(menu);
}

void
requestChecks(ProjectInfo *project, const char *packageName)
{
	DialogList *list;
	Package *package, *pkginfo;
	char **items;
	char **checks;
	int count, i;
	
	list = newDialogList(packageName,
		"Select every feature that configure.in should\n"
		"check for."
	);

	package = getPackageWithName(project, packageName);

	if (package != NULL)
	{
		Check *curCheck, *nextCheck;
		
		count = packageToArray(package, &checks);

		/* Delete the package's checks */
		for (curCheck = package->checks;
			 curCheck != NULL;
			 curCheck = nextCheck)
		{
			nextCheck = curCheck->next;
			freeCheck(curCheck);
		}
			
		package->checks    = NULL;
		package->lastCheck = NULL;
		
		pkginfo = loadPackage(packageName, list, checks, count);
	}
	else
	{
		pkginfo = loadPackage(packageName, list, NULL, 0);
	}

	count = dialogRenderCheckList(list, &items);

	freeDialogList(list);

	if (count > 0)
	{
		/* TODO: Use the Package List, when it's implemented. */
		if (package == NULL)
		{
			package = copyPackage(pkginfo);

			for (i = 0; i < count; i++)
			{
				Check *check =
					copyCheck(getCheckWithName(pkginfo, items[i]));

				addCheck(package, check);
			}

			if (project->packages == NULL)
				project->packages = package;

			package->prev = project->lastPackage;

			if (project->lastPackage != NULL)
				project->lastPackage->next = package;

			project->lastPackage = package;
		}
		else
		{
			package->name        = strdup(pkginfo->name);
			package->description = strdup(pkginfo->description);

			for (i = 0; i < count; i++)
			{
				Check *check =
					copyCheck(getCheckWithName(pkginfo, items[i]));

				addCheck(package, check);
			}
		}

		for (i = 0; i < count; i++)
			free(items[i]);

		free(items);
	}
	else
	{
		if (package != NULL)
		{
			if (package->next != NULL)
			{
				if (package->prev == NULL)
				{
					project->packages = package->next;
					package->next->prev = NULL;
				}
				else
				{
					package->prev->next = package->next;
					package->next->prev = package->prev;
				}
			}
			else
			{
				if (package->prev == NULL)
				{
					project->packages = NULL;
					project->lastPackage = NULL;
				}
				else
				{
					package->prev->next = NULL;
				}
			}
		}
	}

	freePackage(pkginfo);
}

